@@ -49,20 +49,37 @@ K_EXPORT_PLUGIN(KateFileTreeFactory(KAboutData("filetree","katefiletreeplugin",k
 
 //BEGIN KateFileTreePlugin
 KateFileTreePlugin::KateFileTreePlugin(QObject* parent, const QList<QVariant>&)
-  : Kate::Plugin ((Kate::Application*)parent)
+  : Kate::Plugin ((Kate::Application*)parent),
+    m_fileCommand(0)
 {
-
+  KTextEditor::CommandInterface* iface =
+  qobject_cast<KTextEditor::CommandInterface*>(Kate::application()->editor());
+  if (iface) {
+    m_fileCommand = new KateFileTreeCommand(this);
+    iface->registerCommand(m_fileCommand);
+  }
 }
 
 KateFileTreePlugin::~KateFileTreePlugin()
 {
   m_settings.save();
+  KTextEditor::CommandInterface* iface =
+  qobject_cast<KTextEditor::CommandInterface*>(Kate::application()->editor());
+  if (iface && m_fileCommand) {
+    iface->unregisterCommand(m_fileCommand);
+  }
 }
 
 Kate::PluginView *KateFileTreePlugin::createView (Kate::MainWindow *mainWindow)
 {
   KateFileTreePluginView* view = new KateFileTreePluginView (mainWindow, this);
   connect(view, SIGNAL(destroyed(QObject*)), this, SLOT(viewDestroyed(QObject*)));
+  connect(m_fileCommand, SIGNAL(showToolView()), view, SLOT(showToolView()));
+  connect(m_fileCommand, SIGNAL(slotDocumentPrev()), view->tree(), SLOT(slotDocumentPrev()));
+  connect(m_fileCommand, SIGNAL(slotDocumentNext()), view->tree(), SLOT(slotDocumentNext()));
+  connect(m_fileCommand, SIGNAL(slotDocumentFirst()), view->tree(), SLOT(slotDocumentFirst()));
+  connect(m_fileCommand, SIGNAL(slotDocumentLast()), view->tree(), SLOT(slotDocumentLast()));
+  connect(m_fileCommand, SIGNAL(switchDocument(QString)), view->tree(), SLOT(switchDocument(QString)));
   m_views.append(view);
 
   return view;
@@ -156,8 +173,8 @@ KateFileTreePluginView::KateFileTreePluginView (Kate::MainWindow *mainWindow, Ka
   // init console
   kDebug(debugArea()) << "BEGIN: mw:" << mainWindow;
 
-  QWidget *toolview = mainWindow->createToolView (plug,"kate_private_plugin_katefiletreeplugin", Kate::MainWindow::Left, SmallIcon("document-open"), i18n("Documents"));
-  m_fileTree = new KateFileTree(toolview);
+  m_toolView = mainWindow->createToolView (plug,"kate_private_plugin_katefiletreeplugin", Kate::MainWindow::Left, SmallIcon("document-open"), i18n("Documents"));
+  m_fileTree = new KateFileTree(m_toolView);
   m_fileTree->setSortingEnabled(true);
 
   connect(m_fileTree, SIGNAL(activateDocument(KTextEditor::Document*)),
@@ -186,6 +203,7 @@ KateFileTreePluginView::KateFileTreePluginView (Kate::MainWindow *mainWindow, Ka
   connect(dm, SIGNAL(documentWillBeDeleted(KTextEditor::Document*)),
           this, SLOT(documentClosed(KTextEditor::Document*)));
 
+  connect(m_documentModel,SIGNAL(triggerViewChangeAfterNameChange()),this,SLOT(viewChanged()));
   m_fileTree->setModel(m_proxyModel);
 
   m_fileTree->setDragEnabled(false);
@@ -223,6 +241,7 @@ KateFileTreePluginView::~KateFileTreePluginView ()
 
   // clean up tree and toolview
   delete m_fileTree->parentWidget();
+  // delete m_toolView;
   // and TreeModel
   delete m_documentModel;
 }
@@ -237,6 +256,11 @@ KateFileTreeProxyModel *KateFileTreePluginView::proxy()
   return m_proxyModel;
 }
 
+KateFileTree *KateFileTreePluginView::tree()
+{
+  return m_fileTree;
+}
+
 void KateFileTreePluginView::documentOpened(KTextEditor::Document *doc)
 {
   kDebug(debugArea()) << "open" << doc;
@@ -325,11 +349,29 @@ void KateFileTreePluginView::activateDocument(KTextEditor::Document *doc)
   mainWindow()->activateView(doc);
 }
 
+void KateFileTreePluginView::showToolView()
+{
+  mainWindow()->showToolView(m_toolView);
+  m_toolView->setFocus();
+}
+
+void KateFileTreePluginView::hideToolView()
+{
+  mainWindow()->hideToolView(m_toolView);
+  mainWindow()->centralWidget()->setFocus();
+}
+
+void KateFileTreePluginView::switchDocument(const QString &doc)
+{
+  m_fileTree->switchDocument(doc);
+}
+
 void KateFileTreePluginView::showActiveDocument()
 {
   // hack?
   viewChanged();
-  // FIXME: make the tool view show if it was hidden
+  // make the tool view show if it was hidden
+  showToolView();
 }
 
 bool KateFileTreePluginView::hasLocalPrefs()
@@ -379,6 +421,96 @@ void KateFileTreePluginView::writeSessionConfig(KConfigBase* config, const QStri
 
   g.sync();
 }
-//ENDKateFileTreePluginView
+//END KateFileTreePluginView
+
+//BEGIN KateFileTreeCommand
+KateFileTreeCommand::KateFileTreeCommand(QObject *parent)
+  : QObject(parent), KTextEditor::Command()
+{
+}
+
+const QStringList& KateFileTreeCommand::cmds()
+{
+    static QStringList sl;
+
+    if (sl.empty()) {
+     sl << "ls"
+        << "b" << "buffer"
+        << "bn" << "bnext" << "bp" << "bprevious"
+        << "tabn" << "tabnext" << "tabp" << "tabprevious"
+        << "bf" << "bfirst" << "bl" << "blast"
+        << "tabf" << "tabfirst" << "tabl" << "tablast";
+    }
+
+    return sl;
+}
+
+bool KateFileTreeCommand::exec(KTextEditor::View * /*view*/, const QString &cmd, QString & /*msg*/)
+{
+    // create list of args
+    QStringList args(cmd.split(' ', QString::KeepEmptyParts));
+    QString command = args.takeFirst(); // same as cmd if split failed
+    QString argument = args.join(QString(' '));
+
+    if (command == "ls") {
+      emit showToolView();
+    } else if (command == "b" || command == "buffer") {
+      emit switchDocument(argument);
+    } else if (command == "bp" || command == "bprevious" ||
+               command == "tabp" || command == "tabprevious") {
+      emit slotDocumentPrev();
+    } else if (command == "bn" || command == "bnext" ||
+               command == "tabn" || command == "tabnext") {
+      emit slotDocumentNext();
+    } else if (command == "bf" || command == "bfirst" ||
+               command == "tabf" || command == "tabfirst") {
+      emit slotDocumentFirst();
+    } else if (command == "bl" || command == "blast" ||
+               command == "tabl" || command == "tablast") {
+      emit slotDocumentLast();
+    }
+
+    return true;
+}
+
+bool KateFileTreeCommand::help(KTextEditor::View * /*view*/, const QString &cmd, QString &msg)
+{
+    if (cmd == "b" || cmd == "buffer") {
+        msg = i18n("<p><b>b,buffer &mdash; Edit document N from the document list</b></p>"
+                   "<p>Usage: <tt><b>b[uffer] [N]</b></tt></p>");
+        return true;
+    } else if (cmd == "bp" || cmd == "bprevious" ||
+               cmd == "tabp" || cmd == "tabprevious") {
+        msg = i18n("<p><b>bp,bprev &mdash; previous buffer</b></p>"
+                   "<p>Usage: <tt><b>bp[revious] [N]</b></tt></p>"
+                   "<p>Goes to <b>[N]</b>th previous document (\"<b>b</b>uffer\") in document list. </p>"
+                   "<p> <b>[N]</b> defaults to one. </p>"
+                   "<p>Wraps around the start of the document list.</p>");
+        return true;
+    } else if (cmd == "bn" || cmd == "bnext" ||
+               cmd == "tabn" || cmd == "tabnext") {
+        msg = i18n("<p><b>bn,bnext &mdash; switch to next document</b></p>"
+                   "<p>Usage: <tt><b>bn[ext] [N]</b></tt></p>"
+                   "<p>Goes to <b>[N]</b>th next document (\"<b>b</b>uffer\") in document list."
+                   "<b>[N]</b> defaults to one. </p>"
+                   "<p>Wraps around the end of the document list.</p>");
+        return true;
+    } else if (cmd == "bf" || cmd == "bfirst" ||
+               cmd == "tabf" || cmd == "tabfirst") {
+        msg = i18n("<p><b>bf,bfirst &mdash; first document</b></p>"
+                   "<p>Usage: <tt><b>bf[irst]</b></tt></p>"
+                   "<p>Goes to the <b>f</b>irst document (\"<b>b</b>uffer\") in document list.</p>");
+        return true;
+    } else if (cmd == "bl" || cmd == "blast" ||
+               cmd == "tabl" || cmd == "tablast") {
+        msg = i18n("<p><b>bl,blast &mdash; last document</b></p>"
+                   "<p>Usage: <tt><b>bl[ast]</b></tt></p>"
+                   "<p>Goes to the <b>l</b>ast document (\"<b>b</b>uffer\") in document list.</p>");
+        return true;
+    }
+
+    return false;
+}
+//END KateFileTreeCommand
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
