@@ -44,6 +44,7 @@
 #include <qclipboard.h>
 #include <qtextstream.h>
 #include <qfile.h>
+#include <qtimer.h>
 
 DataOutputWidget::DataOutputWidget(QWidget *parent)
 : QWidget(parent)
@@ -97,6 +98,7 @@ DataOutputWidget::DataOutputWidget(QWidget *parent)
 
   layout->addWidget(toolbar);
   layout->addLayout(m_dataLayout);
+  layout->setContentsMargins(0, 0, 0, 0);
 
   setLayout(layout);
 }
@@ -121,7 +123,7 @@ void DataOutputWidget::showQueryResultSets(QSqlQuery &query)
 
   m_isEmpty = false;
 
-  resizeColumnsToContents();
+  QTimer::singleShot(0, this, SLOT(resizeColumnsToContents()));
 
   raise();
 }
